<?php
namespace BookmePro\Lib\Payment;

use BookmePro\Lib;

/**
 * Class Mollie
 */
class Mollie
{
    // Array for cleaning Mollie request
    public static $remove_parameters = array( 'bookme_pro_action', 'bookme_pro_fid', 'error_msg' );

    public static function renderForm( $form_id, $page_url )
    {
        $userData = new Lib\UserBookingData( $form_id );
        if ( $userData->load() ) {
            $replacement = array(
                '%form_id%' => $form_id,
                '%gateway%' => Lib\Entities\Payment::TYPE_MOLLIE,
                '%response_url%' => esc_attr( $page_url ),
            );
            $form = '<form method="post" class="bookme-pro-%gateway%-form">
                <input type="hidden" name="bookme_pro_fid" value="%form_id%"/>
                <input type="hidden" name="bookme_pro_action" value="mollie-checkout"/>
                <input type="hidden" name="response_url" value="%response_url%"/>
             </form>';
            echo strtr( $form, $replacement );
        }
    }

    /**
     * Handles IPN messages
     */
    public static function ipn()
    {
        $payment_details = self::_getApi()->payments->get( $_REQUEST['id'] );
        Mollie::handlePayment( $payment_details );
    }

    /**
     * Check gateway data and if ok save payment info
     *
     * @param \Mollie_API_Object_Payment $details
     */
    public static function handlePayment( \Mollie_API_Object_Payment $details )
    {
        /** @var Lib\Entities\Payment $payment */
        $payment = Lib\Entities\Payment::query()->where( 'type', Lib\Entities\Payment::TYPE_MOLLIE )
            ->where( 'id', $details->metadata->payment_id )->findOne();
        if ( $details->isPaid() ) {
            // Handle completed card & bank transfers here
            $total    = (float) $payment->getPaid();
            $received = (float) $details->amount;

            if ( $payment->getStatus() == Lib\Entities\Payment::STATUS_COMPLETED
                 || $received != $total
            ) {
                wp_send_json_success();
            } else {
                $payment->setStatus( Lib\Entities\Payment::STATUS_COMPLETED )->save();
                if ( $order = Lib\DataHolders\Booking\Order::createFromPayment( $payment ) ) {
                    Lib\NotificationSender::sendFromCart( $order );
                }
            }
        } elseif ( ! $details->isOpen() && ! $details->isPending() ) {
            /** @var Lib\Entities\CustomerAppointment $ca */
            foreach ( Lib\Entities\CustomerAppointment::query()->where( 'payment_id', $details->metadata->payment_id )->find() as $ca ) {
                $ca->deleteCascade();
            }
            $payment->delete();
        }
        wp_send_json_success();
    }

    /**
     * Redirect to Mollie Payment page, or step payment.
     *
     * @param $form_id
     * @param Lib\UserBookingData $userData
     * @param string $page_url
     */
    public static function paymentPage( $form_id, Lib\UserBookingData $userData, $page_url )
    {
        if ( get_option( 'bookme_pro_pmt_currency' ) != 'EUR' ) {
            $userData->setPaymentStatus( Lib\Entities\Payment::TYPE_MOLLIE, 'error', esc_html__( 'Mollie accepts payments in Euro only.', 'bookme_pro' ) );
            @wp_redirect( remove_query_arg( Lib\Payment\Mollie::$remove_parameters, Lib\Utils\Common::getCurrentPageURL() ) );
            exit;
        }
        list( $total, $deposit ) = $userData->cart->getInfo();
        $coupon  = $userData->getCoupon();
        $payment = new Lib\Entities\Payment();
        $payment->setType( Lib\Entities\Payment::TYPE_MOLLIE )
            ->setStatus( Lib\Entities\Payment::STATUS_PENDING )
            ->setCreated( current_time( 'mysql' ) )
            ->setTotal( $total )
            ->setPaid( $deposit )
            ->save();
        $order = $userData->save( $payment );
        try {
            $api = self::_getApi();
            $mollie_payment = $api->payments->create( array(
                'amount'      => $deposit,
                'description' => $userData->cart->getItemsTitle( 125 ),
                'redirectUrl' => add_query_arg( array( 'bookme_pro_action' => 'mollie-response', 'bookme_pro_fid' => $form_id ), $page_url ),
                'webhookUrl'  => add_query_arg( array( 'bookme_pro_action' => 'mollie-ipn' ), $page_url ),
                'metadata'    => array( 'payment_id' => $payment->getId() ),
                'issuer'      => null
            ) );
            if ( $mollie_payment->isOpen() ) {
                if ( $coupon ) {
                    $coupon->claim();
                    $coupon->save();
                }
                $payment->setDetails( $order, $coupon )->save();
                $userData->setPaymentStatus( Lib\Entities\Payment::TYPE_MOLLIE, 'pending', $mollie_payment->id );
                header( 'Location: ' . $mollie_payment->getPaymentUrl() );
                exit;
            } else {
                self::_deleteAppointments( $order );
                $payment->delete();
                self::_redirectTo( $userData, 'error', esc_html__( 'Mollie error.', 'bookme_pro' ) );
            }
        } catch ( \Exception $e ) {
            self::_deleteAppointments( $order );
            $payment->delete();
            $userData->setPaymentStatus( Lib\Entities\Payment::TYPE_MOLLIE, 'error', $e->getMessage() );
            @wp_redirect( remove_query_arg( Lib\Payment\Mollie::$remove_parameters, Lib\Utils\Common::getCurrentPageURL() ) );
            exit;
        }
    }

    /**
     * @param Lib\DataHolders\Booking\Order $order
     */
    private static function _deleteAppointments( Lib\DataHolders\Booking\Order $order )
    {
        foreach ( $order->getFlatItems() as $item ) {
            $item->getCA()->deleteCascade( true );
        }
    }

    private static function _getApi()
    {
        include_once Lib\Plugin::getDirectory() . '/lib/payment/Mollie/API/Autoloader.php';
        $mollie = new \Mollie_API_Client();
        $mollie->setApiKey( get_option( 'bookme_pro_pmt_mollie_api_key' ) );

        return $mollie;
    }

    /**
     * Notification for customer
     *
     * @param Lib\UserBookingData $userData
     * @param string $status    success || error || processing
     * @param string $message
     */
    private static function _redirectTo( Lib\UserBookingData $userData, $status = 'success', $message = '' )
    {
        $userData->load();
        $userData->setPaymentStatus( Lib\Entities\Payment::TYPE_MOLLIE, $status, $message );
        @wp_redirect( remove_query_arg( Lib\Payment\Mollie::$remove_parameters, Lib\Utils\Common::getCurrentPageURL() ) );
        exit;
    }

    /**
     * Get Mollie Payment
     *
     * @param string $tr_id
     * @return \Mollie_API_Object_Payment
     */
    public static function getPayment( $tr_id )
    {
        $api = self::_getApi();

        return $api->payments->get( $tr_id );
    }

}